// the semi-colon before function invocation is a safety net against concatenated
// scripts and/or other plugins which may not be closed properly.
;(function ( $, window, document, undefined ) {

    // undefined is used here as the undefined global variable in ECMAScript 3 is
    // mutable (ie. it can be changed by someone else). undefined isn't really being
    // passed in so we can ensure the value of it is truly undefined. In ES5, undefined
    // can no longer be modified.

    // window and document are passed through as local variable rather than global
    // as this (slightly) quickens the resolution process and can be more efficiently
    // minified (especially when both are regularly referenced in your plugin).

    // Create the defaults once
    var pluginName = "monitorItem",
        defaults = {
            jabberPresence: -1,
            jabberStatusText: "",
        };

    // The actual plugin constructor
    function Plugin( element, options ) {
        this.element = element;

        // jQuery has an extend method which merges the contents of two or
        // more objects, storing the result in the first object. The first object
        // is generally empty as we don't want to alter the default options for
        // future instances of the plugin
        this.options = $.extend( {}, defaults, options );
        this.updates = options;

        this._defaults = defaults;
        this._name = pluginName;

        this.init();
    }

    Plugin.prototype = {

        init: function() {
            // Place initialization logic here
            // You already have access to the DOM element and
            // the options via the instance, e.g. this.element
            // and this.options
            // you can add more functions like the one below and
            // call them like so: this.yourOtherFunction(this.element, this.options).
            
            this.updateProperties(this.element, this.updates);
        },
        
        // update user propperties
        updateProperties: function(el, updates) {
            if(updates.jabberPresence != undefined) this.updateJabberPresence(el, updates);
            if(updates.jabberStatusText != undefined) this.updateJabberStatusText(el, updates);
            if(updates.phonePresence != undefined) this.updatePhonePresence(el, updates);
        },

        // set jabber icon color
        updateJabberPresence: function(el, updates) {
            $(el).find('.icon-comment').css({
                'color': this.colorForJabberState(updates.jabberPresence)
            });
        },
        
        // set jabber status text
        updateJabberStatusText: function(el, updates) {
            $(el).find('.jabberStateMessage').text(updates.jabberStatusText);
        },
        
        
        // set jabber icon color
        updatePhonePresence: function(el, updates) {
            $(el).find('.icon-phone').css({
                'color': this.colorForPhoneState(updates.phonePresence)
            });
        },
        
        colorForJabberState: function(state){
            var stateColor;
            
            switch(state){
                case -1:
                    stateColor = 'gray';
                    break;
                case 0:
                    stateColor = 'green';
                    break;
                case 1:
                    stateColor = 'red';
                    break;  
                case 2:
                    stateColor = '#4f0404';
                    break;
                case 3:
                    stateColor = 'red';
                    break;
                case 4:
                    stateColor = 'green';
                    break;
                case 5:
                    stateColor = 'gray';
                    break;
                default:
                    stateColor = '#fff';
            }
            
            return stateColor;
        },
        
        colorForPhoneState: function(state){
            var stateColor;
            
            switch(state){
                case 'offline':
                    stateColor = 'gray';
                    break;
                case 'ringing':
                    stateColor = 'yellow';
                    break;
                case 'free':
                    stateColor = 'green';
                    break;  
                case 'busy':
                    stateColor = 'red';
                    break;
                case 'hold':
                    stateColor = 'orange';
                    break;
                case 'online':
                    stateColor = 'green';
                    break;
                default:
                    stateColor = 'gray';
            }
            
            return stateColor;
        },
    };

    // A really lightweight plugin wrapper around the constructor,
    // preventing against multiple instantiations
    $.fn[pluginName] = function ( options ) {
        return this.each(function () {
            if (!$.data(this, "plugin_" + pluginName)) {
                $.data(this, "plugin_" + pluginName, new Plugin( this, options ));
            } else {
                return $.data(this, "plugin_" + pluginName).updateProperties(this, options);
            }
        });
    };

})( jQuery, window, document );