(function($){

    // attach to jQuery
    $.extend({

        // export WebSocket = $.WebSocket
        WebSocket: function(url, protocol, options)
        {
            var defaults = {
                url: url,
                http: null,
                enableProtocols: false,
                enablePipes: false,
                debug: false,
                logData: false,
                pass: '',
                user: '',
                uuid: '',
            };
            
            // overwrite (append) to option defaults
            options = $.extend(
                {},
                defaults,
                options
            );
            
            const CONNECTING = 0;
            const OPEN       = 1;
            const CLOSING    = 2;
            const CLOSED     = 3;
            
            // the function table to store references to callbacks
            var _functionTable = {
                onopen:    function()  {connectionEstablished('a')},
                onerror:   function(e) {},
                onclose:   function()  {},
                onmessage: function(e) {messageRecived(e)},
                send:      function(d) { _ws._send(d); }
            };
            
            
            var subscribe = function(channel, data, fn){
                if(options.debug) console.log('Subscribed for: ' + channel);
                if(!this.channels[channel]) this.channels[channel] = [];
                this.channels[channel].push({ context : this, callback : fn });
                return this;
             };
             
             var publish = function(channel){
                  if(!this.channels[channel]) return false;
                  var args = Array.prototype.slice.call(arguments, 1);
                  for(var i = 0, l = this.channels[channel].length; i < l; i++){
                       if(options.debug) console.log('Fire event: ' + channel);
                       var subscription = this.channels[channel][i];
                       //subscription.callback.apply(subscription.context.args);
                       subscription.callback.apply(subscription.context.args, args);
                  };
                  return this;
             };
             
             var command = function(data){
                if(options.debug) console.log('Send command: ' + data.type);
                if(options.logData) console.log(data);
                                  
                 var msg = JSON.stringify(data);
                 this.send(msg);
             }
            
            
/**
             * Returns skeleton of a WebSocket object
             *
             * @param {String} url The url connect to (resource or path ...)
             * @param {String} id  The optional Id of the pipe (unique-identifier)
             *
             * @returns {Object} A fresh pipe (emulated WebSocket)
             * @private
             */
            function _getWebSocketSkeleton(url)
            {
                return {
                    autenticated: false,
                    commandQueue: [],
                    type: 'WebSocket',                  // CUSTOM type     String type of object
                    //uid: _token(),                      // CUSTOM uid      an unique Id used to identify the ws
                    //native: isNative,                   // CUSTOM native   status as boolean
                    readyState: CONNECTING,             // NATIVE default  status is 0 = CONNECTING
                    bufferedAmount: 0,                  // NATIVE integer  currently buffered data in bytes
                    url: url,                           // NATIVE url      The url to connect to

                    send: function(data) {},            // NATIVE send()   sending "data" to server
                    start: function() {},               // CUSTOM start()  used as custom trigger for opening
                    close: function() {},               // NATIVE close()  closes the connection

                    onopen: function() {connectionEstablished()},              // NATIVE EVENT
                    onerror: function(e) {},            // NATIVE EVENT
                    onclose: function() {},             // NATIVE EVENT
                    onmessage: function(e) {},          // NATIVE EVENT

                    protocols: {},                      // CUSTOM          container for protocols in user defined order
                    pipes: {},                          // CUSTOM          container for all registered pipes
                    multiplexed: false,                 // CUSTOM          multiplexed status of this object
                    channels : {},
                    publish : publish,
                    subscribe : subscribe,
                    command : command,
                    sendAuthenticatedCommand : sendAuthenticatedCommand,

                    /**
                     * Export: registerPipe()
                     *
                     * Creates a new pipe and return pipe-reference.
                     *
                     * This method creates a "pipe" which is in fact an emulated WebSocket
                     * (the same emulation as used in older browsers). These "pipes" are
                     * used as a logical connection within a physical connection to the Server
                     *
                     * Pipe 1 -> O==\     (WebSocket)    /==O -> Pipe 1
                     * Pipe 2 -> O==========================O -> Pipe 2
                     * Pipe 3 -> O==/                    \==O -> Pipe 3
                     *
                     * So we can use those Pipes in the same way like the original WebSocket object.
                     * You can use the same events what enables you and your app to use less
                     * connections between server and client and use different endpoints for your
                     * services too.
                     *
                     * @param {String} url      The url to connect to
                     * @param {String} protocol The protocol to use
                     * @param {Object} options  Custom options to pass through
                     *
                     * @returns {Object} the instance created
                     */
                    registerPipe: function(url, protocol, options) {
                        var p = new _pipe(url);
                        this.multiplexed = true;

                        // we iterate the functionTable and use the events for injecting our hooks
                        for (event in _functionTable) {
                            // inject all hooks except "send"
                            if (event != 'send') {
                                _injectHook(event, p);
                            }
                        }

                        // try:
                        // return ws.pipes[p.id] = p = new _pipe(url, id);
                        var r = this.pipes[p.uid] = $.extend(p, options).start();
                        return r;
                    },
                    unregisterPipe: function(id) {
                        this.pipes[id] = null;
                        (this.pipes.length === 0) ? this.multiplexed = false : void(0);
                        // timer?
                    },
                    registerProtocol: function(name, callback) {
                        var p = new _protocol(name, callback);
                        return this.protocols[name] = p;
                    },
                    unregisterProtocol: function(name) {
                        this.protocols[name] = null;
                    },
                    extension: null,
                    protocol: null,
                    reason: null,
                    binaryType: null,
                    
                    generateUUID:function () {
                        var d = new Date().getTime();
                        var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
                        var r = (d + Math.random() * 16) % 16 | 0;
                        d = Math.floor(d / 16);
                        return (c == 'x' ? r : (r & 0x3 | 0x8)).toString(16);
                    });
                    return uuid;
                    }
                    
                };
                
                
            }
            
            
            function _dispatchProtocol(e, direction, ws)
            {
                // give object event from protocol to protocol
                for (var protocol in ws.protocols) {
                    e = ws.protocols[protocol].callback(e, direction);
                }

                // return dirty object
                return e;
            };
            
            
            /**
             * private: _proxy
             *
             * This is the proxy between an intercepted call and calls
             * the user defined callback
             *
             * @param {String} trigger The trigger (event-name) to dispatch
             * @param {Object} e       The event object to dispatch
             *
             * @returns The result of dispatch (depends on operation!)
             * @private
             */
            function _proxy(trigger, e)
            {
                return _functionTable[trigger](e);
            };
            
            
            
            /**
             * private: _injectHook
             *
             * Injects hooks into a given WebSocket object
             *
             * This method injects the hooks for the given name of an event (event)
             * into a given WebSocket object (ws).
             *
             * @param {String} event The name of the event to hook
             * @param {Object} ws    An instance of a WebSocket to patch (inject to)
             *
             * @returns void
             * @private
             */
            function _injectHook(event, ws)
            {
                //
                if (event == 'send') {
                    ws._send = ws.send;
                }

                // install our proxy method for intercepting events
                ws[event] = function(e) {
                    var uid;

                    if (event == 'onmessage') {
                        // apply registered protocols first
                        e = _dispatchProtocol(e, 'i', ws);

                        // where to get info -> multiplexed?
                        // if multiplexed look into package for uid
                        if (ws.multiplexed) {
                            // extract uid
                            uid = /"uid"\:\s"([a-z0-9A-z]*)"/.exec(e.data)[1];
                        }

                        if (uid !== undefined) {
                            $(ws.pipes[uid]).trigger(e);
                        } else {
                            // dispatch to all pipes
                            for (pipe in ws.pipes) {
                                $(ws.pipes[pipe]).trigger(
                                    e
                                );
                            }
                        }
                    }

                    (event == 'send') ? e = _dispatchProtocol(e, 'o', ws) : null;

                    _proxy(event, e);
                };

                // override setter with custom hook to fetch user defined callbacks
                window.WebSocket.prototype.__defineSetter__(
                   event,
                    function(v) {
                        _functionTable[event] = v;
                    }
                );

                // override getter with custom hook to fetch user defined callbacks
                window.WebSocket.prototype.__defineGetter__(
                    event,
                    function()  {
                        return _functionTable[event];
                    }
                );
            };
            
            
            
            /**
             * private: _extend
             *
             * Copies all properties from source to destination object as
             * long as they not exist in destination.
             *
             * This method is intend to ...
             *
             * @param {Object} source      The object to copy from
             * @param {Object} destination The object to copy to
             *
             * @returns {Object} The object destination with new properties set
             * @private
             */
            function _extend(source, destination) {
              for (property in source) {
                if (!destination[property]) {
                  destination[property] = source[property];
                }
              }

              return destination;
            }
            
            
            
            /**
             * private: _getWebSocket
             *
             * Returns a WebSocket-Object (native or emulated)
             *
             * @returns {Object} WebSocket
             * @private
             */
            function _getWebSocket(url)
            {
              var ws = true;

                if (window.WebSocket) {
                    ws = new WebSocket(url);
                } else {
                    console.log('WebSocket not supported');
                }

                // extend it with our additions and return
                return _extend(
                    _getWebSocketSkeleton(url),
                    ws
                );
            };
            
            
            /***********************************************************************************************************
             *
             *  MAIN()
             *
             **********************************************************************************************************/

            // get WebSocket object (either native or emulated)
                var _ws = _getWebSocket(options.url);

            // we iterate the functionTable and use the events for injecting our hooks
            for (event in _functionTable) {
                _injectHook(event, _ws);
            }
            
            
            
            
            
            function connectionEstablished(){
                if(options.debug) console.log('WebSocket server connected');
            
                var userData = {user: options.user, pass: options.pass};
                authenticate(userData);
                
            }
            
            function authenticate(userData){
                if(options.debug) console.log('Try to authenticate user:' + userData.user + ' password:' + userData.pass);
            
                //this.readyState = AUTHENTICATING;
                
                var msg = {
                    data: userData,
                    name: 'login',
                    type: 'command',
                    uuid: _ws.generateUUID()
                } 
                _ws.command(msg);
            }
            
            function sendAuthenticatedCommand(data, fn)
            {
                if (_ws.autenticated) {
                    _ws.command(data);
                } else {
                    _ws.commandQueue.push(data);
                }
            }
            
            function messageRecived(e){
                var msg = jQuery.parseJSON(e.data);
                
                if(options.debug) console.log("WebSocket message recived");
                if(options.logData) console.log(msg);
                
                // response recived
                if(msg.type == 'response'){                    
                    if(msg.name == 'login'){
                        _ws.autenticated = true;
                        $.each(_ws.commandQueue, function(i,data) {
                            _ws.sendAuthenticatedCommand(data);
                            delete _ws.commandQueue[i];
                        });
                        
                        if(options.debug) console.log(msg.data.status + ': ' + msg.data.status_text);
                    }
                    else{
                        //_ws.publish(msg.event.typ, msg.event.data);    
                    }
                        
                }
                
                // event recived
                if(msg.type == 'event'){
                    if(options.debug){
                        console.log('Event recived');
                    }
                    _ws.publish(msg.name, msg.data);    
                }
                
                
                
            }

            // return WebSocket
            return _ws;
        }
    });
})(jQuery);