<?php
/**
 * CSort class file.
 *
 * @author Qiang Xue <qiang.xue@gmail.com>
 * @link http://www.yiiframework.com/
 * @copyright Copyright &copy; 2008-2011 Yii Software LLC
 * @license http://www.yiiframework.com/license/
 */

/**
 * CSort represents information relevant to sorting.
 *
 * When data needs to be sorted according to one or several attributes,
 * we can use CSort to represent the sorting information and generate
 * appropriate hyperlinks that can lead to sort actions.
 *
 * CSort is designed to be used together with {@link CActiveRecord}.
 * When creating a CSort instance, you need to specify {@link modelClass}.
 * You can use CSort to generate hyperlinks by calling {@link link}.
 * You can also use CSort to modify a {@link CDbCriteria} instance by calling {@link applyOrder} so that
 * it can cause the query results to be sorted according to the specified
 * attributes.
 *
 * In order to prevent SQL injection attacks, CSort ensures that only valid model attributes
 * can be sorted. This is determined based on {@link modelClass} and {@link attributes}.
 * When {@link attributes} is not set, all attributes belonging to {@link modelClass}
 * can be sorted. When {@link attributes} is set, only those attributes declared in the property
 * can be sorted.
 *
 * By configuring {@link attributes}, one can perform more complex sorts that may
 * consist of things like compound attributes (e.g. sort based on the combination of
 * first name and last name of users).
 *
 * The property {@link attributes} should be an array of key-value pairs, where the keys
 * represent the attribute names, while the values represent the virtual attribute definitions.
 * For more details, please check the documentation about {@link attributes}.
 *
 * @property string $orderBy The order-by columns represented by this sort object.
 * This can be put in the ORDER BY clause of a SQL statement.
 * @property array $directions Sort directions indexed by attribute names.
 * The sort direction. Can be either CSort::SORT_ASC for ascending order or
 * CSort::SORT_DESC for descending order.
 *
 * @author Qiang Xue <qiang.xue@gmail.com>
 * @version $Id: CSort.php 3554 2012-02-08 16:31:30Z alexander.makarow $
 * @package system.web
 */
class CPhonalisaSort extends CSort
{
   

    /**
     * Resolves the attribute label for the specified attribute.
     * This will invoke {@link CActiveRecord::getAttributeLabel} to determine what label to use.
     * If the attribute refers to a virtual attribute declared in {@link attributes},
     * then the label given in the {@link attributes} will be returned instead.
     * @param string $attribute the attribute name.
     * @return string the attribute label
     */
    public function resolveLabel($attribute)
    {

        $definition=$this->resolveAttribute($attribute);
        if(is_array($definition))
        {
            if(isset($definition['label']))
                return $definition['label'];
        }
        else if(is_string($definition))
            $attribute=$definition;
        if($this->modelClass!==null)
            return PhonalisaRecord::model($this->modelClass)->getAttributeLabel($attribute);
        else
            return $attribute;
    }
// 
    /**
     * Returns the real definition of an attribute given its name.
     *
     * The resolution is based on {@link attributes} and {@link CActiveRecord::attributeNames}.
     * <ul>
     * <li>When {@link attributes} is an empty array, if the name refers to an attribute of {@link modelClass},
     * then the name is returned back.</li>
     * <li>When {@link attributes} is not empty, if the name refers to an attribute declared in {@link attributes},
     * then the corresponding virtual attribute definition is returned. Starting from version 1.1.3, if {@link attributes}
     * contains a star ('*') element, the name will also be used to match against all model attributes.</li>
     * <li>In all other cases, false is returned, meaning the name does not refer to a valid attribute.</li>
     * </ul>
     * @param string $attribute the attribute name that the user requests to sort on
     * @return mixed the attribute name or the virtual attribute definition. False if the attribute cannot be sorted.
     */
    public function resolveAttribute($attribute)
    {
        if($this->attributes!==array())
            $attributes=$this->attributes;
        else if($this->modelClass!==null)
            $attributes=PhonalisaRecord::model($this->modelClass)->attributeNames();
        else
            return false;
        foreach($attributes as $name=>$definition)
        {
            if(is_string($name))
            {
                if($name===$attribute)
                    return $definition;
            }
            else if($definition==='*')
            {
                if($this->modelClass!==null && PhonalisaRecord::model($this->modelClass)->hasAttribute($attribute))
                    return $attribute;
            }
            else if($definition===$attribute)
                return $attribute;
        }
        return false;
    }
    public function link($attribute,$label=null,$htmlOptions=array()){

        if($label===null) {
            $label=$this->resolveLabel($attribute);
        }

        if(($definition=$this->resolveAttribute($attribute))===false) {
            return $label;
        }


        $directions=$this->getDirections();
        if(isset($directions[$attribute]))
        {
            $class=$directions[$attribute] ? 'desc' : 'asc';
            if(isset($htmlOptions['class'])) {

                $htmlOptions['class'].=' '.$class;
            } else {
                $htmlOptions['class']=$class;
            }
            $descending=!$directions[$attribute];
            unset($directions[$attribute]);
        
        } else if(is_array($definition) && isset($definition['default'])) {

            $descending=$definition['default']==='desc';
        } else {

            $descending=false;
        }

        if($this->multiSort) {

            $directions=array_merge(array($attribute=>$descending),$directions);
        } else {

            $directions=array($attribute=>$descending);
        }

        $url=$this->createUrl(Yii::app()->getController(),$directions);
        $link = $this->createLink($attribute,$label,$url,$htmlOptions);

        return $link;        
    }

        public function getOrderBy($criteria=null)
    {
        
        $directions=$this->getDirections();
        if(empty($directions))
            return is_string($this->defaultOrder) ? $this->defaultOrder : '';
        else
        {
            if($this->modelClass!==null)
     //           $schema=CActiveRecord::model($this->modelClass)->getDbConnection()->getSchema();
            $orders=array();
            foreach($directions as $attribute=>$descending)
            {
                $definition=$this->resolveAttribute($attribute);
                if(is_array($definition))
                {
                    if($descending)
                        $orders[]=isset($definition['desc']) ? $definition['desc'] : $attribute.' DESC';
                    else
                        $orders[]=isset($definition['asc']) ? $definition['asc'] : $attribute;
                }
                else if($definition!==false)
                {
                    $attribute=$definition;
                    if(isset($schema))
                    {
                        if(($pos=strpos($attribute,'.'))!==false)
                            $attribute=$schema->quoteTableName(substr($attribute,0,$pos)).'.'.$schema->quoteColumnName(substr($attribute,$pos+1));
                        else
                            $attribute=($criteria===null || $criteria->alias===null ? CActiveRecord::model($this->modelClass)->getTableAlias(true) : $criteria->alias).'.'.$schema->quoteColumnName($attribute);
                    }
                    $orders[]=$descending?$attribute.' DESC':$attribute;
                }
            }
            return implode(', ',$orders);
        }
    }
}